Re-exports
pub use scale_set::ScaleSet;
Modules
- alpha
- color
- continuous
- datetime
- discrete
- format
- Label formatting functions for scale breaks.
Analogous to R’s
scalespackage (comma, percent, dollar, scientific, number, SI, ordinal, bytes). - gradient
- gradient_
n - grey
- linetype
- linetype_
manual - manual
- modified
- A color scale that wraps another scale and adjusts the lightness of its
mapped colors — the backing implementation for
after_scalecolor derivations (e.g. a fill that is a darker version of the mapped color). - palettes
- scale_
set - sec_
axis - shape
- shape_
manual - size
- steps
- transform
- util
Traits
- Scale
- Trait for scales that map data values to visual properties.