Module summary2d

Module summary2d 

Structs

StatSummary2d
2-D binned summary (analogous to R’s stat_summary_2d): bins x/y into a grid and applies a summary function to the z values in each cell, emitting xmin/xmax/ymin/ymax/fill (fill = the per-cell summary) for geom_bin2d.