Modules
- area
- bar
- bin2d
- blank
- boxplot
- bracket
- candlestick
- Financial OHLC charts:
geom_candlestick(body + wick) andgeom_ohlc(bar with open/close ticks). Mapxplus theopen/high/low/closeaesthetics; the y scale trains on the high–low range. - censor
- Censor marks for survival curves:
+glyphs at the (time, survival) points where observations were censored (ggsurvfit’sadd_censor_mark). - col
- contour
- cooks
- Cook’s-distance contours for a residuals-vs-leverage plot (R’s
plot.lm(which = 5)): withx= leveragehandy= standardized residualr, Cook’s distance isD = r² h / (p (1 − h)), so the points withD = levellie onr = ±√(level · p · (1 − h) / h). - count
- crossbar
- curve
- density
- density2d
- dotplot
- errorbar
- errorbarh
- Horizontal error bars (
geom_errorbarh): a line from(xmin, y)to(xmax, y)with vertical end caps — horizontal confidence intervals in a coefficient forest or a time-to-event plot. - freqpoly
- hex
- histogram
- jitter
- line
- linerange
- path
- point
- pointrange
- polygon
- raster
- rect
- refline
- Reference lines:
geom_hline,geom_vline,geom_abline. - repel
- Repelling text labels —
geom_text_repel/geom_label_repel(R’s ggrepel). - ribbon
- rug
- segment
- smooth
- spoke
- step
- text
- tile
- violin
Structs
- Geom
Params - Fixed (non-mapped) visual parameters for a geom.
Traits
- Geom
- Trait for geometric objects that draw data on the plot.