A color scale that wraps another scale and adjusts the lightness of its
mapped colors — the backing implementation for after_scale color
derivations (e.g. a fill that is a darker version of the mapped color).
Structs
- Scale
Color Modified - Wraps a source scale, exposing it under a different (target) aesthetic and lightness-adjusting every mapped color. All non-color behaviour delegates to the inner scale.